%% 1D grating example
%% This driver performs a scan over angle of incidence

%% vacuum wavelengths
lambda0 = 193e-9;

%% inclination angle range in degree of the incoming field
thetas = linspace(-15,15,11);

%% azimuthal angle in degree
phi = 20;

%% fix pattern lower/upper pattern width
keys.pattern_width_bottom = 200;
keys.pattern_width_top = 190;

%% perform the scan and store -1,0,+1 diffraction orders for
%% incident s and p polarization (transmission and reflection)
diffraction_order_s_t = [];
diffraction_order_p_t = [];
diffraction_order_s_r = [];
diffraction_order_p_r = [];

for iRun = 1 : length(thetas)
  theta = thetas(iRun);
  keys.lambda0 = lambda0;
  keys.thetaPhi = [theta phi];

  %% solve the project 
  %% use temporary project folder (option 'temporary'='yes')
  results = jcmwave_solve('project.jcmp', keys, 'temporary', 'yes');

  %% gather transmission results
  fourierTransform_t = results{2};
  powerFluxes_t = jcmwave_convert2powerflux(fourierTransform_t);
  for iOrder=-1 : 1
    for jOrder=-1 : 1
      iMode = find((fourierTransform_t.N1==iOrder) & ...
		   (fourierTransform_t.N2==jOrder));
      if isempty(iMode)
	diffraction_order_s_t(iOrder+2, jOrder+2, iRun) = nan;
	diffraction_order_p_t(iOrder+2, jOrder+2, iRun) = nan;
      else
	diffraction_order_s_t(iOrder+2, jOrder+2, iRun) = powerFluxes_t.PowerFluxDensity{1}(iMode, 3);
	diffraction_order_p_t(iOrder+2, jOrder+2, iRun) = powerFluxes_t.PowerFluxDensity{2}(iMode, 3);
      end 
    end
  end


  %% gather reflection results
  fourierTransform_r = results{3};
  powerFluxes_r = jcmwave_convert2powerflux(fourierTransform_r);
  for iOrder=-1 : 1
    for jOrder=-1 : 1
      iMode = find((fourierTransform_r.N1==iOrder) & ...
		   (fourierTransform_r.N2==jOrder));
      if isempty(iMode)
	diffraction_order_s_r(iOrder+2, jOrder+2, iRun) = nan;
        diffraction_order_p_r(iOrder+2, jOrder+2, iRun) = nan;
      else
        diffraction_order_s_r(iOrder+2, jOrder+2, iRun) = powerFluxes_r.PowerFluxDensity{1}(iMode, 3);
        diffraction_order_p_r(iOrder+2, jOrder+2, iRun) = powerFluxes_r.PowerFluxDensity{2}(iMode, 3);
      end
    end 
  end
end

%% plot results
fig = figure(1);
clf
set(fig, 'Units', 'normalized', 'Position', [0.2,0.1,0.4,0.8]);

labels = {};
for iOrder=-1 : 1
  for jOrder=-1 : 1
    labels{end+1} = sprintf('order = [%i, %i]',iOrder,jOrder);
  end
end

x = sind(thetas);
colors = {'b', 'r', 'g'};
symbols = {'+', 'o', 'x'};

subplot(2,2,1);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_s_t(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('S-Polarisation'); xlim([x(1) x(end)]); 
xlabel('sin(\vartheta)');
ylabel('Transmission');
legend(labels, 'Location', 'Best');
set(gca, 'FontSize', 12)

subplot(2,2,3);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_p_t(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('P-Polarisation'); xlim([x(1) x(end)]); 
xlabel('sin(\vartheta)');
ylabel('Transmission');
set(gca, 'FontSize', 12)
hold off

subplot(2,2,2);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_s_r(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('S-Polarisation'); xlim([x(1) x(end)]); 
xlabel('sin(\vartheta)');
ylabel('Reflection');
set(gca, 'FontSize', 12)

subplot(2,2,4);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_p_r(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('P-Polarisation'); xlim([x(1) x(end)]); 
xlabel('sin(\vartheta)');
ylabel('Reflection');
suptitle('2D square lattice grating, illumination scan');
set(gca, 'FontSize', 12)
hold off

