%% 2D grating example
%% This driver performs a scan over width (at bottom) of the pattern

%% vacuum wavelengths
keys.lambda0 = 193e-9;

%% pattern width scan sample positions 
pattern_widths = linspace(150,250,10);

%% fix the incidence angle
keys.thetaPhi = [0 0];

%% perform the scan and store -1,0,+1 diffraction orders for
%% incident s and p polarization (transmission and reflection)
diffraction_order_s_t = [];
diffraction_order_p_t = [];
diffraction_order_s_r = [];
diffraction_order_p_r = [];

for iRun = 1 : length(pattern_widths)
  keys.pattern_width_bottom = pattern_widths(iRun);
  keys.pattern_width_top = keys.pattern_width_bottom-10;
  
  %% solve the project 
  %% use temporary project folder (option 'temporary'='yes')
  results = jcmwave_solve('project.jcmp', keys, 'temporary', 'yes');

  %% gather transmission results
  fourierTransform_t = results{2};
  powerFluxes_t = jcmwave_convert2powerflux(fourierTransform_t);
  for iOrder=-1 : 1
    for jOrder=-1 : 1
      iMode = find((fourierTransform_t.N1==iOrder) & ...
		   (fourierTransform_t.N2==jOrder));
      if isempty(iMode)
	diffraction_order_s_t(iOrder+2, jOrder+2, iRun) = nan;
	diffraction_order_p_t(iOrder+2, jOrder+2, iRun) = nan;
      else
	diffraction_order_s_t(iOrder+2, jOrder+2, iRun) = powerFluxes_t.PowerFluxDensity{1}(iMode, 3);
	diffraction_order_p_t(iOrder+2, jOrder+2, iRun) = powerFluxes_t.PowerFluxDensity{2}(iMode, 3);
      end 
    end
  end


  %% gather reflection results
  fourierTransform_r = results{3};
  powerFluxes_r = jcmwave_convert2powerflux(fourierTransform_r);
  for iOrder=-1 : 1
    for jOrder=-1 : 1
      iMode = find((fourierTransform_r.N1==iOrder) & ...
		   (fourierTransform_r.N2==jOrder));
      if isempty(iMode)
	diffraction_order_s_r(iOrder+2, jOrder+2, iRun) = nan;
        diffraction_order_p_r(iOrder+2, jOrder+2, iRun) = nan;
      else
        diffraction_order_s_r(iOrder+2, jOrder+2, iRun) = powerFluxes_r.PowerFluxDensity{1}(iMode, 3);
        diffraction_order_p_r(iOrder+2, jOrder+2, iRun) = powerFluxes_r.PowerFluxDensity{2}(iMode, 3);
      end
    end 
  end
end

%% plot results
fig = figure(1);
clf
set(fig, 'Units', 'normalized', 'Position', [0.2,0.1,0.4,0.8]);

labels = {};
for iOrder=-1 : 1
  for jOrder=-1 : 1
    labels{end+1} = sprintf('order = [%i, %i]',iOrder,jOrder);
  end
end

x = pattern_widths;
colors = {'b', 'r', 'g'};
symbols = {'+', 'o', 'x'};

subplot(2,2,1);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_s_t(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('S-Polarisation');
xlabel('line width [nm]');
ylabel('Transmission');
legend(labels, 'Location', 'Best')
set(gca, 'FontSize', 12)

subplot(2,2,3);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_p_t(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('P-Polarisation');
xlabel('line width [nm]');
ylabel('Transmission');
set(gca, 'FontSize', 12)
hold off

subplot(2,2,2);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_s_r(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('S-Polarisation'); 
xlabel('line width [nm]');
ylabel('Reflection');
set(gca, 'FontSize', 12)

subplot(2,2,4);
hold on
for iOrder=-1 : 1
  for jOrder=-1 : 1
    plot(x,squeeze(diffraction_order_p_r(iOrder+2, jOrder+2, :)),[colors{iOrder+2} symbols{jOrder+2}, '-'], 'LineWidth', 2);
  end
end
title('P-Polarisation'); 
xlabel('line width [nm]');
ylabel('Reflection');
suptitle('2D square lattice grating, pattern width scan');
set(gca, 'FontSize', 12)
hold off

