#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import sys,os
sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))

import jcmwave
import numpy as np
import matplotlib.pyplot as plt
import copy

# Dielectric Meta Surface
# This driver determines the parameters dependency of the transmission
#
# This example is taken from
# Berzins, Jonas, Fabrizio Silvestri, Giampiero Gerini, Frank Setzpfandt, Thomas Pertsch, and Stefan MB Baeumer.
# "Color filter arrays based on dielectric metasurface elements." 
# In Metamaterials XI, vol. 10671, p. 106711F. International Society for Optics and Photonics, 2018.


jcmwave.daemon.shutdown()
jcmwave.daemon.add_workstation(Hostname ='localhost', 
                               Multiplicity = 2, 
                               NThreads = 1)

uol=1e-9
default_keys = {'h_top' : 100.0,
                'h_bottom' : 100.0,
                'h_disk' : 175.0,
                'd_disk':  115.0 ,
                'pitch' : 250.0,
                'slc' : 400.0,
                'fem_degree': 5 }

Si = np.genfromtxt('Si.txt',skip_header=13)

wavelength  = 550.
default_keys['vacuum_wavelength'] = wavelength*uol        
default_keys['n_disk'] = np.interp(wavelength,Si[:,0],Si[:,1])
default_keys['k_disk'] = np.interp(wavelength,Si[:,0],Si[:,2])
default_keys['slc'] = wavelength/3.
default_keys['slc2'] = wavelength/3./1.47
default_keys['slc3'] = wavelength/3./default_keys['n_disk']

diameters = np.arange(50,200,1.25)
heights = np.arange(100,200,1.25)

#create resultbag. Results will be stored in file ./meta_surface.db
resultbag = jcmwave.Resultbag('meta_surface.db')
job_ids = []
for diam in diameters:
    keys = copy.deepcopy(dict(default_keys))
    keys['d_disk']=diam
    job_id = jcmwave.solve('project.jcmp',
                           mode='solve',keys=keys,
                           temporary = True,   
                           resultbag = resultbag)
    job_ids.append(job_id)

for h in heights:
    keys = copy.deepcopy(dict(default_keys))
    keys['h_disk']=h
    job_id = jcmwave.solve('project.jcmp',
                           mode='solve',keys=keys,
                           temporary = True,   
                           resultbag = resultbag)
    job_ids.append(job_id)
    
# here we wait until all jobs are finished
all_results, logs = jcmwave.daemon.wait(job_ids = job_ids, resultbag = resultbag)

#%%
data = np.empty((len(diameters)+len(heights),11))
data[:] = np.nan
all_results = []
for diam in diameters:
    keys = copy.deepcopy(dict(default_keys))
    keys['d_disk']=diam
    data[len(all_results),9]=keys['h_disk']
    data[len(all_results),10]=keys['d_disk']
    all_results.append(resultbag.get_result(keys))
for h in heights:
    keys = copy.deepcopy(dict(default_keys))
    keys['h_disk']=h
    data[len(all_results),9]=keys['h_disk']
    data[len(all_results),10]=keys['d_disk']
    all_results.append(resultbag.get_result(keys))

#%%
# The first JonesMatrix (assumes there only is 1 K-directoin) is extracted and 
# the complex values are stored as pairs of magnitude and phase in a 2x4 array. 
# iF refers to the first incoming direction
def ReadJonesMatrix(scatteringMatrix,iF=0, iK=0):
    J = np.array([[scatteringMatrix['Jones_ps11'][iF][iK], scatteringMatrix['Jones_ps12'][iF][iK]],
                  [scatteringMatrix['Jones_ps21'][iF][iK], scatteringMatrix['Jones_ps22'][iF][iK]]])
    return J
c0   = 299792458
mu0  = 4.*np.pi*1e-7
eps0 = 1./(mu0*c0**2)
Z0 = np.sqrt(mu0/eps0)
area = (default_keys['pitch']*uol)**2
pIn = 1.0*area
for iT,this_result in enumerate(all_results):    
    data[iT,0] =wavelength
    
    # to compute reflection and transmission coeffcients, the power flux densities 
    # in both directions are computed from the corresponding FourierTransforms
    pf = jcmwave.convert2powerflux(this_result[1])
    data[iT,1] = sum(pf['PowerFluxDensity'][0][:,2])
    data[iT,4] = sum(pf['PowerFluxDensity'][1][:,2])

    pf = jcmwave.convert2powerflux(this_result[2])
    data[iT,2] = -sum(pf['PowerFluxDensity'][0][:,2])
    data[iT,5] = -sum(pf['PowerFluxDensity'][1][:,2])

    absorption = this_result[5]['ElectromagneticFieldAbsorption']
    data[iT,3] = absorption[0][2].real/pIn
    data[iT,6] = absorption[1][2].real/pIn
    
    # iK is the indext of the 0th diffraction order
    iK =np.logical_and(this_result[1]['N1']==0,this_result[1]['N2']==0)
    J = ReadJonesMatrix(this_result[4],0,iK)
    data[iT,7] = np.abs(J[0,0])
    data[iT,8] = np.angle(J[0,0])


#%%    
plt.close('all')
fig, (ax1,ax2) = plt.subplots(ncols=2)
ax12=ax1.twinx()
ax22=ax2.twinx()
ax1.set_xlabel('diameter')
ax2.set_xlabel('height')
ax1.set_ylabel(r'$\vert t\vert$')
ax2.set_ylabel(r'$\vert t\vert$')
ax12.set_ylabel(r'$phase [\pi]$')
ax22.set_ylabel(r'$phase [\pi]$')

fig.suptitle('Parameter dependency of the complex transmission coefficient')

index =range(len(diameters))
ax1.plot(data[index,10],data[index,7],label='abs(t)'),
ax12.plot(data[index,10],data[index,8]/np.pi,label=r'$\arg(t)/\pi$',linestyle = '--',color='C1')

lines, labels = ax1.get_legend_handles_labels()
lines2, labels2 = ax12.get_legend_handles_labels()
ax1.legend(lines + lines2, labels + labels2)

index = range(len(diameters),len(diameters)+len(heights))
ax2.plot(data[index,9],data[index,7],label='abs(t)')
ax22.plot(data[index,9],data[index,8]/np.pi,label=r'$\arg(t)/\pi$',linestyle = '--',color='C1')

lines, labels = ax2.get_legend_handles_labels()
lines2, labels2 = ax22.get_legend_handles_labels()
ax2.legend(lines + lines2, labels + labels2)

plt.show()
