#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Author: Martin Hammerschmidt
"""
import sys,os
sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))

import jcmwave
import numpy as np
import matplotlib.pyplot as plt
import copy
jcmwave.daemon.shutdown()
jcmwave.daemon.add_workstation(Hostname='localhost',                  
                               Multiplicity=1,
                               NThreads=8)
uol=1e-9
default_keys = {'h_top' : 100.0,
                'h_bottom' : 100.0,
                'h_disk' : 175.0,
                'd_disk':  115.0 ,
                'pitch' : 250.0,
                'slc' : 400.0,
                'fem_degree': 5 }

Si = np.genfromtxt('Si.txt',skip_header=13)

wavelengths = np.arange(400,710,2.5)

resultbag = jcmwave.Resultbag('meta_surface.db')
job_ids = []
for iW,wavelength in enumerate(wavelengths):
    keys = copy.deepcopy(dict(default_keys))
    keys['vacuum_wavelength'] = wavelength*uol        
    keys['n_disk'] = np.interp(wavelength,Si[:,0],Si[:,1])
    keys['k_disk'] = np.interp(wavelength,Si[:,0],Si[:,2])
    keys['slc'] = wavelength/3.
    keys['slc2'] = wavelength/3./1.47
    keys['slc3'] = wavelength/3./keys['n_disk']
    
    job_id = jcmwave.solve('project.jcmp',
                           mode='solve',keys=keys,
                           temporary = True,   
                           resultbag = resultbag)
    job_ids.append(job_id)

# here we wait until all jobs are finished
all_results, logs = jcmwave.daemon.wait(job_ids = job_ids, resultbag = resultbag)

all_results = []
for iW,wavelength in enumerate(wavelengths):
    keys = copy.deepcopy(dict(default_keys))
    keys['vacuum_wavelength'] = wavelength*uol        
    keys['n_disk'] = np.interp(wavelength,Si[:,0],Si[:,1])
    keys['k_disk'] = np.interp(wavelength,Si[:,0],Si[:,2])
    keys['slc'] = wavelength/3.
    keys['slc2'] = wavelength/3./1.47
    keys['slc3'] = wavelength/3./keys['n_disk']
    all_results.append(resultbag.get_result(keys))
    

#%%
c0   = 299792458
mu0  = 4.*np.pi*1e-7
eps0 = 1./(mu0*c0**2)
Z0 = np.sqrt(mu0/eps0)
area = (default_keys['pitch']*uol)**2
# plane wave input is normalized using PowerFlux=UnitNormal 
pIn = 1.0*area
data = np.empty((len(wavelengths),10))
data[:] = np.nan
for iW,wavelength in enumerate(wavelengths):
    this_result = all_results[iW]
    data[iW,0] =wavelength
    
    # to compute reflection and transmission coeffcients, the power flux densities 
    # in both directions are computed from the corresponding FourierTransforms
    pf = jcmwave.convert2powerflux(this_result[1])
    data[iW,1] = sum(pf['PowerFluxDensity'][0][:,2])
    data[iW,4] = sum(pf['PowerFluxDensity'][1][:,2])

    pf = jcmwave.convert2powerflux(this_result[2])
    data[iW,2] = -sum(pf['PowerFluxDensity'][0][:,2])
    data[iW,5] = -sum(pf['PowerFluxDensity'][1][:,2])

    absorption = this_result[5]['ElectromagneticFieldAbsorption']
    data[iW,3] = absorption[0][2].real/pIn
    data[iW,6] = absorption[1][2].real/pIn
    

#%%
plt.close('all')
plt.clf()
plt.plot(data[:,0],data[:,1],label='R')
plt.plot(data[:,0],data[:,2],label='T')
plt.plot(data[:,0],data[:,3],label='A')
# plot energy conservation
#plt.plot(data[:,0],data[:,1]+data[:,2]+data[:,3],label='R+T+A')
plt.legend()

plt.xlabel('wavelength [nm]')
plt.ylabel('R/T/A')
plt.title('Spectral properties of the array')


#%%
# The first JonesMatrix (assumes there only is 1 K-direction) is extracted and 
# the complex values are stored as pairs of magnitude and phase in a 2x4 array. 
# iF refers to the first incoming direction
def ReadJonesMatrix(scatteringMatrix,iF=0, iK=0):
    J = np.array([[scatteringMatrix['Jones_ps11'][iF][iK], scatteringMatrix['Jones_ps12'][iF][iK]],
                  [scatteringMatrix['Jones_ps21'][iF][iK], scatteringMatrix['Jones_ps22'][iF][iK]]])
    return J

for iW,wavelength in enumerate(wavelengths):
    this_result = all_results[iW]
    

    # iK is the index of the 0th diffraction order
    iK =np.logical_and(this_result[1]['N1']==0,this_result[1]['N2']==0)
    J = ReadJonesMatrix(this_result[4],0,iK)
    data[iW,7] = np.abs(J[0,0])
    data[iW,8] = np.angle(J[0,0])

    
    
fig, ax1 = plt.subplots()
ax2=ax1.twinx()

ax1.set_xlabel('wavelength [nm]')
ax1.set_ylabel(r'$\vert t\vert$')
ax2.set_ylabel(r'$phase [\pi]$')

plt.title('Spectrum of the complex transmission coefficient')

ax1.plot(data[:,0],data[:,7],label='abs(t)'),
ax2.plot(data[:,0],data[:,8]/np.pi,label=r'$\arg(t)/\pi$',linestyle = '--',color='C1')

lines, labels = ax1.get_legend_handles_labels()
lines2, labels2 = ax2.get_legend_handles_labels()
ax1.legend(lines + lines2, labels + labels2)

plt.show()
