#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys,os
try:
    sys.path.append(os.path.join(os.getenv('JCMROOT'), 'ThirdPartySupport', 'Python'))
except:
    pass


import jcmwave
import numpy as np
jcmwave.daemon.shutdown()

# Dielectric Meta Surface
# This simple driver the simulation
#
# This example is taken from
# Berzins, Jonas, Fabrizio Silvestri, Giampiero Gerini, Frank Setzpfandt, Thomas Pertsch, and Stefan MB Baeumer.
# "Color filter arrays based on dielectric metasurface elements." 
# In Metamaterials XI, vol. 10671, p. 106711F. International Society for Optics and Photonics, 2018.

##define default key values
uol=1e-9
keys = {'h_top' : 100.0,
        'h_bottom' : 100.0,
        'h_disk' : 175.0,
        'd_disk':  115.0 ,
        'pitch' : 250.0,
        'slc' : 400.0,
        'fem_degree': 5 }


results = jcmwave.solve('project.jcmp',
                        mode='solve')

#%% process the computed data to extract reflection and transmission

# The first JonesMatrix (assumes there only is 1 K-direction) is extracted and 
# the complex values are stored as pairs of magnitude and phase in a 2x4 array. 
# iF refers to the first incoming direction
def ReadJonesMatrix(scatteringMatrix,iF=0, iK=0):
    J = np.array([[scatteringMatrix['Jones_ps11'][iF][iK], scatteringMatrix['Jones_ps12'][iF][iK]],
                  [scatteringMatrix['Jones_ps21'][iF][iK], scatteringMatrix['Jones_ps22'][iF][iK]]])
    return J

c0   = 299792458
mu0  = 4.*np.pi*1e-7
eps0 = 1./(mu0*c0**2)
Z0 = np.sqrt(mu0/eps0)
area = (keys['pitch']*uol)**2   
pIn = 1.0*area

# to compute reflection and transmission coeffcients, the power flux densities 
# in both directions are computed from the corresponding FourierTransforms
pf = jcmwave.convert2powerflux(results[1])
R_p = sum(pf['PowerFluxDensity'][0][:,2])
R_s = sum(pf['PowerFluxDensity'][1][:,2])

pf = jcmwave.convert2powerflux(results[2])
T_p = -sum(pf['PowerFluxDensity'][0][:,2])
T_s = -sum(pf['PowerFluxDensity'][1][:,2])

absorption = results[5]['ElectromagneticFieldAbsorption']
A_p = absorption[0][2].real/pIn
A_s = absorption[1][2].real/pIn

# iK is the index of the 0th diffraction order
iK = np.logical_and(results[1]['N1']==0,results[1]['N2']==0)
J = ReadJonesMatrix(results[4],0,iK)
t_p = np.abs(J[0,0])
phase_p = np.angle(J[0,0])

#%% 
print('\nTotal transmission for p-polarized incoming field: T={:2.4g}%'.format(R_p*100.0))
print('Total reflection for p-polarized incoming field:   R={:2.4g}%'.format(T_p*100.0))
print('Total absorption for p-polarized incoming field:   A={:2.4g}%'.format(A_p*100.0))

print('\nTotal transmission for s-polarized incoming field: T={:2.4g}%'.format(R_p*100.0))
print('Total reflection for s-polarized incoming field:   R={:2.4g}%'.format(T_p*100.0))
print('Total absorption for s-polarized incoming field:   A={:2.4g}%'.format(A_p*100.0))

print('\nThe transmitted light undergoes a phase shift of {:0.1g} pi'.format(phase_p[0]/np.pi))

