% EUV line mask: simulate reflection
% setup according to  Proc. SPIE Vol. 8083 (2011) 80831B
% preprint at: https://arxiv.org/pdf/1105.6052v1.pdf


% geometry definitions 
keys.uol = 1e-9; % unit of length
keys.pitch_x = 176;
keys.cd_bottom = 88; % critical dimension / line width
keys.line_swa = 85; % sidewall angle

keys.corner_rounding_r = 5; % top corner rounding
keys.height_offset_air = 5;
keys.height_absorber = 40;
keys.height_buffer = 20;
keys.height_oxide_layer = 1.1;
keys.height_capping_layer = 11.5;
keys.multilayer_mirror_h1_mo = 2.42; 
keys.multilayer_mirror_h2_si = 4.48; 
keys.multilayer_n_layers = 60;
keys.slc = 7; % maximum mesh element size

% material definitions: refractive indices
keys.n_1 = 1; % air
keys.n_2 = 0.93368 + 0.03791i; % Cr absorber
keys.n_3 = 0.97468 + 0.01261i; % SiO2 buffer and oxide layer
keys.n_4 = keys.n_3;
keys.n_5 = 1.00024 + 0.00182i; % Si capping layer
keys.n_6 = 0.92373 + 0.00610i; % Mo multilayer mirror
keys.n_7 = keys.n_5; % Si multilayer mirror
keys.n_10 = 0.97908; % a_SiO2 substrate without absorption

% source definitions: 
keys.vacuum_wavelength = 13.4e-9;
keys.theta = 4; % inclination angle
keys.phi = 0; % rotation angle

% numerical definitions
keys.fem_degree = 3;
keys.n_refinement_steps = 1;
keys.info_level = 3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION 
% using Domain-Decomposition approach:
% the layout does not include the DBR multilayer
% instead the DBR multilayer is defined as LayeredMedia in the 
% file materials.jcm

keys.layout_includes_multilayer = false;
results = jcmwave_solve('project.jcmp', keys);

% display diffraction order intensities
n = results{2}.N1;
intensities_s = sum(abs(results{2}.ElectricFieldStrength{1}).^2, 2);
intensities_p = sum(abs(results{2}.ElectricFieldStrength{2}).^2, 2);
figure(1); semilogy(n, intensities_s, '-or', n, intensities_p, '-ob'); 
xlabel('Diffraction order'); ylabel('Intensity (S/P)'); legend('S', 'P'); 
title('EUV line mask, reflection');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION 
% NOT using Domain-Decomposition approach:
% the layout does include the DBR multilayer
% 
% check that this (standard) approach yields the same results as the 
% domain-decomposition approach

simulate_2d_full = false;

if (simulate_2d_full)
  keys.layout_includes_multilayer = true;
  results_2d_full = jcmwave_solve('project.jcmp', keys);

  % display diffraction order intensities
  n = results_2d_full{2}.N1;
  intensities_s = sum(abs(results_2d_full{2}.ElectricFieldStrength{1}).^2, 2);
  intensities_p = sum(abs(results_2d_full{2}.ElectricFieldStrength{2}).^2, 2);
  figure(2); semilogy(n, intensities_s, '-or', n, intensities_p, '-ob'); 
  xlabel('Diffraction order'); ylabel('Intensity (S/P)'); legend('S', 'P'); 
  title('EUV line mask, reflection (no domain decomposition)');
  
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







