%% Setup JCMsolve and define constants
clear variables; close all;
jcm_root = getenv('JCMROOT');
addpath(fullfile(jcm_root, 'ThirdPartySupport', 'Matlab'));

% constants
c0   = 299792458;
keys.uol = 1.e-9;

%% Setup multilayer or 1D photonic crystal geometry
% The multilayer setup definitions "PhCBook1" and "PhCBook2" are taken from
% Chapter 4 of Joannopoulos, John D., et al. Photonic crystals: molding the flow of light.,
% Princeton university press, 2011.
% The "DBR" is a distributed Bragg reflector setup with layer thicknesses adjusted to the refractive indizes
setting = 'DBR';

pitch = 500; % arbitrary choice
if strcmp(setting,'PhCBook1')
    keys.n = [sqrt(13) 1];
    keys.thicknesses = pitch*[.5 .5 ];    
elseif strcmp(setting,'PhCBook2')
    keys.n = [sqrt(13) 1];
    keys.thicknesses = pitch*[.2 .8 ];
elseif strcmp(setting,'DBR')
    keys.n = [ 3.1 1.5];
    keys.thicknesses = pitch./keys.n;
else
    error('unknown setting');
end
keys.domainIds = 1:length(keys.thicknesses);

%% Solver settings
keys.slcs = pitch./keys.n/3; % use 3 segments per wavelength
keys.n_evs = 8;
keys.ev_guess = 2*pi*c0/pitch/keys.uol;

%% Band diagram computation
% discretize Brillouin zone with nBlochSamples points
a= 2*pi/sum(keys.thicknesses)/keys.uol; % lattice constant
nBlochSamples=11;
nBlochSamples=2*floor(nBlochSamples/2)+1;
blochPath = linspace(-a/2,a/2,nBlochSamples);

modes = NaN(keys.n_evs,nBlochSamples); results=cell(1,nBlochSamples);
for ii=1:nBlochSamples
    keys.kBloch = [blochPath(ii) 0 0 ];
    results{ii} = jcmwave_solve('--solve','project.jcmp',keys);
    % store eigenmode frequencies
    modes(:,ii) = results{ii}{1}.eigenvalues.eigenmode;
end

%% Plot dimensionless band diagram and compute size of band gap
figure(1);clf
dimensionless_modes = real(modes)*sum(keys.thicknesses)*keys.uol/(2*pi*c0);
plot(blochPath/a,dimensionless_modes,'bx-')
hold on;
plot(blochPath([1 end])/a,dimensionless_modes(:,[1 end]),'k--')
for ii=1:keys.n_evs/2
    h=patch(blochPath([1 end end 1])/a,[dimensionless_modes(2*ii-1,1) dimensionless_modes(2*ii-1,end) dimensionless_modes(2*ii,end) dimensionless_modes(2*ii,1)],'y','FaceAlpha',.3);
end
center_id= floor(nBlochSamples/2)+1; % select center of Brillouin zone
for ii=1:keys.n_evs/2-1
    if diff(dimensionless_modes(2*ii+(0:1),center_id))>.1
        h=patch(blochPath([1 end end 1])/a,dimensionless_modes(2*ii+[0 0 1 1],center_id),'y','FaceAlpha',.3);
    end
end
xlim([-.5 .5]);xlabel('wavevector: ka / 2\pi');
ylim([0 2]);ylabel('frequency: \omega a / 2\pi c');
gap_size = diff(dimensionless_modes(1:2,end));
title(sprintf('bandgap size : %0.2g%%',gap_size*100))
hold off;

