clear variables; close all;

% constants
c0   = 299792458; 
mu0  = 4*pi*1e-7; 
eps0 = 1/(mu0*c0^2);
Z0   = sqrt(mu0/eps0);

% parameters
keys.uol = 1.e-9;              

%% Setup solar cell geometry
thicknesses = [ 600 ;... % tco thickness
                300 ];   % silicon thickness
ag_thickness = 300;            
 
% wavelength settings
wavelength_interval = [350 800];
sampling_rate  = 20;
wavelengths = linspace(wavelength_interval(1), wavelength_interval(2),round(diff(wavelength_interval)/sampling_rate));

% load material data for Si from file
Si = importdata('Si.txt',' ', 13); Si = Si.data;
Si(:,1) =  Si(:,1)*1e-9;   [foo,I] = unique(Si(:,1));   Si = Si(I,:);

% refractive indices
keys.n(1) = 1.52;         % glass
keys.n(2) = 2.1+.01*1i;   % tco
keys.n(3) = 1;            % silicon
keys.n(4) = .15016 + 3.4727i; % Ag

% solver settings
keys.precision = 1e-4;

% incoming flux for normalization
p_in = 0.5/Z0*keys.n(1); 

%% Run wavelength scan
absorption = NaN(6,length(wavelengths));
reflection = NaN(2,length(wavelengths));
transmission = reflection;
absorption_via_flux = reflection;
reflection_via_flux = reflection;
unknowns = reflection;
total_time = reflection;
percentages = NaN(20,length(wavelengths));

for ii = 1:length(wavelengths)
    keys.vacuum_wavelength = wavelengths(ii)*keys.uol;
    keys.n(3) = interp1(Si(:,1), Si(:,2), keys.vacuum_wavelength, 'linear') ... % n 
                + 1i*interp1(Si(:,1), Si(:,3), keys.vacuum_wavelength, 'linear'); % k

    %% Solve with PEC boundary condition
    keys.slcs = max(real(keys.vacuum_wavelength*1e9./keys.n(2:3))/4,3);    
    keys.thicknesses = thicknesses;
    results = jcmwave_solve('--solve', 'project.jcmp', keys,'jcmt_pattern','PEC');
    
    %% Solve with PML
    keys.slcs = max(real(keys.vacuum_wavelength*1e9./keys.n(2:end))/4,3);
    keys.thicknesses = [thicknesses; ag_thickness]; % additional Ag layer defined
    results_pml = jcmwave_solve('--solve', 'project.jcmp', keys,'jcmt_pattern','pml');

    %% Absorbed energy
    omega = 2*pi*c0/keys.vacuum_wavelength;         
    absorption(1:2,ii) = results{2}.ElectromagneticFieldAbsorption{1}/p_in;
    absorption(4:6,ii) = results_pml{2}.ElectromagneticFieldAbsorption{1}/p_in;       
    
    %% Flux outgoing
    energy_flux = cell2mat(results{3}.ElectromagneticFieldEnergyFlux);
    energy_flux_pml = cell2mat(results_pml{3}.ElectromagneticFieldEnergyFlux);
    
    ref_index = find(results{3}.DomainIdFirst==2 & results{3}.DomainIdSecond==1);
    reflection(1,ii) = abs(real(energy_flux(ref_index))/p_in); 
    ref_index = find(results_pml{3}.DomainIdFirst==2 & results_pml{3}.DomainIdSecond==1);
    reflection(2,ii) = abs(real(energy_flux_pml(ref_index))/p_in); 

    energy_flux = cell2mat(results{4}.ElectromagneticFieldEnergyFlux);   
    energy_flux_pml = cell2mat(results_pml{4}.ElectromagneticFieldEnergyFlux);
    
    trans_index = find(results_pml{4}.DomainIdFirst==4 & results_pml{4}.DomainIdSecond==1);
    transmission(2,ii) = abs(real(energy_flux_pml(trans_index))/p_in); 
 
    %% Statistics
    total_time(1,ii) = results{1}.computational_costs.TotalTime(end);
    unknowns(1,ii)   = results{1}.computational_costs.Unknowns(end);
    mem(1,ii)        = results{1}.computational_costs.TotalMemory_GB(end);
    total_time(2,ii) = results_pml{1}.computational_costs.TotalTime(end);
    unknowns(2,ii)   = results_pml{1}.computational_costs.Unknowns(end);
    mem(2,ii)        = results_pml{1}.computational_costs.TotalMemory_GB(end);
    
    for jj=1:10
        percentages(jj,ii) = results{1}.computational_costs.(sprintf('FEDegree%d_Percentage',jj))(end);
    end
    for jj=1:10
        percentages(10+jj,ii) = results_pml{1}.computational_costs.(sprintf('FEDegree%d_Percentage',jj))(end);
    end
    
end

%% Plot results
figure(1)
suptitle('Absorption spectra of flat layer stack')
subplot(3,1,1)
area(wavelengths,[absorption([2 1],:);reflection(1,:)]') % change ordering of materials to have Si absorption at the bottom
legend('Si','TCO','Reflection');
xlabel('wavelength'); ylabel('Absorption / R / T');
ylim([0 1.05])
title('PEC boundary condition')

subplot(3,1,2)
area(wavelengths,[absorption([5 4 6],:);reflection(2,:);transmission(2,:)]') % change ordering of materials to have Si absorption at the bottom
legend('Si','TCO','Ag', 'Reflection','Transmission');
xlabel('wavelength'); ylabel('Absorption / R / T');
ylim([0 1.05])
title('PML setup with additional Ag layer in stack')

subplot(3,1,3)
plot(wavelengths,absorption(2,:),'rx-',wavelengths,1-reflection(1,:),'k--',wavelengths,absorption(5,:),'rx-',wavelengths,1-reflection(2,:),'k--')
plot(wavelengths,sum(absorption(1:2,:)),'rx-',wavelengths,1-reflection(1,:),'k--',wavelengths,sum(absorption(4:6,:)),'rx-',wavelengths,1-reflection(2,:),'k--')
plot(wavelengths,sum(absorption(1:2,:)),'rx-',wavelengths,1-reflection(1,:),'k--')
semilogy(wavelengths,abs(1-sum(absorption(1:2,:))-reflection(1,:)),'rx-',...
          wavelengths,abs(1-sum(absorption(4:6,:))-reflection(2,:)-transmission(2,:)),'kx-')
legend('PEC boundary setup', 'PML setup');
xlabel('wavelength'); ylabel('energy conservation error');
title('energy conservation error')
