clear variables; close all;
jcm_root = getenv('JCMROOT');
addpath(fullfile(jcm_root, 'ThirdPartySupport', 'Matlab'));

% constants
c0   = 299792458; 
mu0  = 4*pi*1e-7; 
eps0 = 1/(mu0*c0^2);
Z0   = sqrt(mu0/eps0);

% parameters
keys.uol = 1.e-9;              

%% Setup solar cell geometry
% load interface and shift to zero integral to keep material volume
% constant
interface   = load('interface.txt'); % simple list of x y pairs in keys.uol scaling
keys.pitch  = diff(interface([1 end],1));
shift       = trapz(interface(:,1),interface(:,2))/diff((interface([1 end],1)));
interface(:,2)  = interface(:,2)-shift;
interface(:,1)  = interface(:,1) - interface(1,1) - keys.pitch/2;
keys.interface  = interface.';

keys.thicknesses = [ 600 ;... % tco thickness
                     300 ];   % silicon thickness
keys.h_offset = max(abs(interface(:,2)))+100;


% wavelength settings
wavelength_interval = [350 800];
sampling_rate  = 10;
wavelengths = linspace(wavelength_interval(1), wavelength_interval(2),round(diff(wavelength_interval)/sampling_rate));

% load material data for Si from file
Si = importdata('Si.txt',' ', 13); Si = Si.data;
Si(:,1) =  Si(:,1)*1e-9;   [foo,I] = unique(Si(:,1));   Si = Si(I,:);

% refractive indices
keys.n(1) = 1.52;         % glass
keys.n(2) = 2.1+.01*1i;   % tco
keys.n(3) = 1;            % silicon

% solver settings
keys.n_refinement_steps    = 1;
keys.n_prerefinement_steps = 0;
keys.precision = 1e-2;
keys.fe_degree = 2;

% incoming flux for normalization
p_in = 0.5/Z0*keys.pitch*keys.uol*keys.n(1); 

%% Run wavelength scan
absorption = NaN(3,length(wavelengths));
reflection = NaN(1,length(wavelengths));
absorption_via_flux = reflection;
reflection_via_flux = reflection;
unknowns = reflection;
total_time = reflection;


for ii = 1:length(wavelengths)
    keys.vacuum_wavelength = wavelengths(ii)*keys.uol;

    keys.n(3) = interp1(Si(:,1), Si(:,2), keys.vacuum_wavelength, 'linear') ... % n 
              + 1i*interp1(Si(:,1), Si(:,3), keys.vacuum_wavelength, 'linear'); % k

    %% Solve
    results = jcmwave_solve('--solve', 'project.jcmp', keys,'jcmt_pattern','simple_setup');

    %% Absorbed energy    
    absorption(:,ii) = results{2}.ElectromagneticFieldAbsorption{1}/p_in;
    
    %% Fourier Transform
    e_field = cell2mat(results{3}.ElectricFieldStrength);
    km = norm(results{3}.K(ceil(end/2),:));
    angles = real(acosd(-results{3}.K(:,2)/km));
    refl = sum(conj(e_field).*e_field,2).'*cosd(angles);   
    reflection(ii) = sum(refl,1)*.5/Z0*keys.n(1)*keys.pitch*keys.uol/p_in;
    
    %% Flux outgoing
    energy_flux = cell2mat(results{4}.ElectromagneticFieldEnergyFlux);
    index = results{4}.DomainIdFirst == 2 & results{4}.DomainIdSecond == 3;
    absorption_via_flux(ii) = real(energy_flux(index))/p_in;

    index = results{4}.DomainIdFirst == 2 & results{4}.DomainIdSecond == 1;
    reflection_via_flux(ii) = real(p_in+energy_flux(index))/p_in; 

    %% Statistics
    total_time(ii) = results{1}.computational_costs.TotalTime(end);
    unknowns(ii)   = results{1}.computational_costs.Unknowns(end);
    mem(ii)        = results{1}.computational_costs.TotalMemory_GB(end);
end

%% Plot results
figure(1)
subplot(2,1,1)
plot(wavelengths,absorption(3,:),'rx-',wavelengths,1-reflection,'k--')
legend('Absorption', '1-Reflection');
xlabel('wavelength'); ylabel('Absorption / 1-R');

subplot(2,1,2)
area(wavelengths,[absorption([3 2],:);reflection]') % change ordering of materials to have Si absorption at the bottom
legend('Si','TCO', '1-Reflection');
xlabel('wavelength'); ylabel('Absorption / 1-R');
ylim([0 1.01])

%% Solar Spectrum
solar_spectrum_data = importdata('ASTMG173.csv',';',2); % Data is reformatted from ASTMG173.xls available at http://rredc.nrel.gov/solar/spectra/am1.5/
planck = 6.62606957*10^-34;                                     % Planck's constant
energy = planck*c0/keys.uol./solar_spectrum_data.data(:,1);     % photo energy
photon_number = solar_spectrum_data.data(:,3)./energy*1e-4;     % number of photons in cm-2 nm^-1 s-1
spectrum_weight = interp1(solar_spectrum_data.data(:,1),photon_number,wavelengths);

eC = 1.602176565*10^-19;    % elementary charge
currentDensity      = eC*trapz(wavelengths,[absorption.*repmat(spectrum_weight,3,1)]')*1000;
currentDensity(3)   = eC*trapz(wavelengths,reflection.*spectrum_weight)*1000;

fprintf('\n Implied photo current densities in (%dnm, %dnm):\n %g mAcm-2 (TCO)\n %g mAcm-2 (Si)\n %g mAcm-2(R)\n\n',round(wavelengths([1 end])),    currentDensity)
