%% 3D quantum dot emitter with micro lens (exploiting rotational symmetry)
%% The extracted light is coupled into an optical fiber
%% This driver performs a scan of the diameter of the lens and the fiber core

lens_diameters=[3000 : 100 : 3800];
core_radii=[500, 1000]; 
job_ids = [];
for core_radius=core_radii
  keys.core_radius=core_radius;
  for lens_diameter=lens_diameters
    keys.lens_diameter=lens_diameter;
    %% solve the fiber mode problem before emission problem
    fprintf('\n*** Solving for core radius: %2.4g lens_diameter: %2.4g%', core_radius, lens_diameter);
    results=jcmwave_solve({'fiber_modes/project.jcmp', './project.jcmp'}, keys, 'temporary', 'no');
    emission=results{2}{2}.DipoleEmission(1);
    emission_background=results{2}{2}.DipoleBulkEmission(1);
    mode_coefficients=results{2}{3}.ModeCoefficients{1};

    %% sum up coupling of dipole source to fundamental fiber modes. 
    mode_coupling=sum(abs(mode_coefficients.^2));
 
    %% the mode coupling efficiency is the quotient between the dipole emission
    %% and the energy in the waveguide modes
    coupling_efficiency = mode_coupling/emission;
    %% the Purcell factor is the quotient between the dipole emission in the cavity
    %% and the emission of a dipole in a homogeneous space with the background material
    %% at the dipole position    
    purcell_factor = emission/emission_background;

    fprintf('\nPurcell factor: %2.6g Coupling efficiency: %2.6g%%', purcell_factor, coupling_efficiency*100);

    idr=find(core_radii==core_radius);
    idl=find(lens_diameters==lens_diameter);
    coupling_efficiencies(idr,idl) = coupling_efficiency;
    purcell_factors(idr,idl) = purcell_factor;

  end
end

figure('pos', [10 10 1000 600])
subplot(1,2,1)
plot(lens_diameters, coupling_efficiencies, '-+', 'LineWidth', 2);
xlabel('lens diameter [nm]', 'FontSize', 14);
ylabel('coupling efficiency', 'FontSize', 14);
legend(num2str(core_radii','fiber core radius %.1f nm'),'Location','southeast');
subplot(1,2,2)
plot(lens_diameters, purcell_factors, '-+', 'LineWidth', 2);
xlabel('lens diameter [nm]', 'FontSize', 14);
ylabel('Purcell factor', 'FontSize', 14);
